from django.conf import settings


FEINCMS_GRID_TOTAL_COLUMNS = getattr(settings, 'FEINCMS_GRID_TOTAL_COLUMNS', 12)
FEINCMS_GRID_FIELDS = getattr(settings, 'FEINCMS_GRID_FIELDS', ('grid_columns', 'grid_push', 'grid_pull', 'grid_offset', 'grid_extra_classes'))


from django import template

from feincms.templatetags.feincms_tags import _render_content
from feincms_grid.settings import FEINCMS_GRID_TOTAL_COLUMNS


register = template.Library()


class NoGridValuesException(Exception):
    pass


class GridBuilder(object):
    items = []
    row_columns = 0
    current_row = []
    return_string = ''

    def __init__(self, request, context={}):
        self.request = request
        self.context = context
        self.row_columns = 0
        self.items = []
        self.current_row = []
        self.return_string = ''

    def add(self, item):
        self.items.append(item)

    def clear_queue(self, orphan=None):
        self.row_columns = 0
        self.current_row = []
        if orphan:
            self.current_row.append(orphan)

    def render_row(self, content, wrap=True):
        if type(content) == list:
            content = ''.join(content)
        if wrap:
            self.return_string += "<div class='row'>%s</div>" % content
        else:
            self.return_string += content
        
    def render(self):
        for item in self.items:
            rendered = _render_content(item, request=self.request, context=self.context)
            # In case you try to render a contenttype that isn't a subclass of GridContent.
            try:
                if item.grid_columns:

                    # Add current column to the total combined row columns.
                    item_columns = item.grid_columns
                    if item.grid_offset:
                        item_columns += item.grid_offset

                    # Can't add, starting a new row with the item.
                    if self.row_columns + item_columns > FEINCMS_GRID_TOTAL_COLUMNS:
                        self.render_row(self.current_row)
                        self.clear_queue(rendered)

                    # Add it to the row.
                    else:
                        self.row_columns += item_columns
                        self.current_row.append(rendered)

                        # If this row is full, start a new one.
                        if self.row_columns == FEINCMS_GRID_TOTAL_COLUMNS:
                            self.render_row(self.current_row)
                            self.clear_queue()

                # No defined grid_columns
                else:
                    raise NoGridValuesException

            # Do it the old fashined way.
            except (AttributeError, NoGridValuesException):
                self.render_row(rendered, wrap=False)
                self.clear_queue()

        # Make sure to render any left overs.
        if self.current_row:
            self.render_row(self.current_row)

        # Clear out this object.
        self.items = []
        self.clear_queue()

        return self.return_string


@register.simple_tag(takes_context=True)
def feincms_grid_render_region(context, feincms_object, region, request=None):
    """
    {% feincms_grid_render_region feincms_page "main" request %}
    """

    grid = GridBuilder(request, context)

    for content in getattr(feincms_object.content, region):
        grid.add(content)

    return grid.render()



from django import template

from feincms_grid.settings import FEINCMS_GRID_FIELDS


register = template.Library()


@register.filter
def post_process_fieldsets(fieldset, remove=None, keep=None):
    """
    Removes a few fields from FeinCMS admin inlines, those being
    ``id``, ``DELETE`` and ``ORDER`` currently.

    Additionally, it ensures that dynamically added fields (i.e.
    ``ApplicationContent``'s ``admin_fields`` option) are shown.
    """
    # abort if fieldset is customized
    if fieldset.model_admin.fieldsets:
        return fieldset

    fields_to_include = set(fieldset.form.fields.keys())
    for f in ('id', 'DELETE', 'ORDER'):
        fields_to_include.discard(f)

    if remove:
        for f in FEINCMS_GRID_FIELDS:
            fields_to_include.discard(f.strip())

    if keep:
        for f in list(fields_to_include):
            if f not in keep:
                fields_to_include.discard(f)

    def _filter_recursive(fields):
        ret = []
        for f in fields:
            if isinstance(f, (list, tuple)):
                # Several fields on one line
                sub = _filter_recursive(f)
                # Only add if there's at least one field left
                if sub:
                    ret.append(sub)
            elif f in fields_to_include:
                ret.append(f)
                fields_to_include.discard(f)
        return ret

    new_fields = _filter_recursive(fieldset.fields)
    # Add all other fields (ApplicationContent's admin_fields) to
    # the end of the fieldset
    for f in fields_to_include:
        new_fields.append(f)

    fieldset.fields = new_fields
    return fieldset


@register.filter
def only_grid_fields(form):
    new_fieldsets = []
    for fieldset in form:
        fieldset = post_process_fieldsets(fieldset, keep=FEINCMS_GRID_FIELDS)
        new_fieldsets.append(fieldset.fields)
    form.fieldsets = [(None, {'fields': new_fieldsets})]
    if not new_fieldsets[0]:
        return None
    return form



from django.db import models

from settings import FEINCMS_GRID_TOTAL_COLUMNS


class GridContent(models.Model):
    grid_columns = models.PositiveSmallIntegerField('Columns', blank=True, null=True, help_text='How many columns of the total (%s) this content will span.' % FEINCMS_GRID_TOTAL_COLUMNS)
    grid_offset = models.PositiveSmallIntegerField('Offset', blank=True, null=True, help_text='Offset to the right this many columns.')
    grid_push = models.PositiveSmallIntegerField('Push', blank=True, null=True, help_text='Source ordering - push this content this many columns to the right.')
    grid_pull = models.PositiveSmallIntegerField('Pull', blank=True, null=True, help_text='Source ordering - pull this content this many columns to the left.')
    grid_extra_classes = models.CharField('Extra classes', max_length=255, blank=True, help_text='Manual class definitions, e.g.: centered, large-X, small-X, etc.')

    class Meta:
        abstract = True

    def render(self, **kwargs):
        """
        Render implimentation that attempts to add CSS grid classes to the final output.
        """

        classes = []
        if self.grid_columns:
        	classes.append('large-%s columns' % self.grid_columns)
        if self.grid_offset:
        	classes.append('large-offset-%s' % self.grid_offset)
        if self.grid_push:
        	classes.append('large-push-%s' % self.grid_push)
        if self.grid_pull:
        	classes.append('large-pull-%s' % self.grid_pull)
        if self.grid_extra_classes:
        	for c in self.grid_extra_classes.split(' '):
        		classes.append(c)

        return "<div class='%s'>%s</div>" % (' '.join(classes), self.rendered)


from setuptools import setup, find_packages
from feincms_grid import __version__ as version
import os

README = os.path.join(os.path.dirname(__file__), 'README')

with open(README) as fobj:
    long_description = fobj.read()

setup(name="feincms-grid",
    version=version,
    description="Integrate CSS grids with FeinCMS contenttypes",
    long_description=long_description,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Topic :: Software Development',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],
    keywords='django feincms foundation grid',
    author='Joshua Jonah',
    author_email='joshuajonahcom@gmail.com',
    url='http://github.com/joshuajonah/feincms_grid',
    license='BSD',
    packages=find_packages(),
    install_requires=['Django>=1.7', 'FeinCMS>=1.10.1'],
    include_package_data=True,
    zip_safe=False
)